// dendodb.pas
// This file is part of the DendoPlayer.
//
//     Copyright(c) 2008 PT2K <PT2K@mymail.to>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

unit DendoDB;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Contnrs;

type
  TDendoStateType = (dstNotDownloaded, dstDownloading, dstDownloaded);

const
  DendoStateStrs: array[TDendoStateType] of String =
    ('', '@', '+');

type
  TDendoFieldType = (
    dftState, dftStar  , dftTitle, dftGenre, dftAuthor  , dftDate,
    dftTime , dftMember, dftSize , dftID   , dftFileName
  );

type
  TDendoRecordItem = array[TDendoFieldType] of String;

type

  { TDendoDBRecord }

  TDendoDBRecord = class
  private
    FFields: TDendoRecordItem;
  protected
    function Get(Field: TDendoFieldType): String;
    function GetText(): String;
    procedure Put(Field: TDendoFieldType; Value: String);
    procedure SetText(Value: String);
  public
    property Fields[Field: TDendoFieldType]: String read Get write Put; default;
    property Text: String read GetText write SetText;
    procedure Assign(Rec: TDendoDBRecord);
    function Dup(): TDendoDBRecord;
  end;
  
type

  { TDendoDatabase }

  TDendoDatabase = class(TObjectList)
  private
  protected
    function GetItem(Index: Integer): TDendoDBRecord;
//    procedure Notify(Ptr: Pointer; Action: TListNotification); override;
  public
    property Items[Index: Integer]: TDendoDBRecord read GetItem; default;
//    function Add(Item: TDendoDBRecord): Integer;
    procedure Assign(DB: TDendoDatabase);
    procedure AssignToStrings(Strings: TStrings);
    function IndexOf(Field: TDendoFieldType; Value: String): Integer;
    procedure LoadFromFile(FileName: TFileName);
    procedure MergeDB(OldDB: TDendoDatabase);
    procedure SaveToFile(FileName: TFileName);
    procedure SortBy(Field: TDendoFieldType; IsAscending: Boolean);
  end;

implementation

{ TDendoDBRecord }

function TDendoDBRecord.Get(Field: TDendoFieldType): String;
begin
  Result := FFields[Field];
end;

function TDendoDBRecord.GetText(): String;
var
  Field: TDendoFieldType;
  SL: TStringList;
begin
  SL := TStringList.Create();
  try
    for Field := Low(TDendoFieldType) to High(TDendoFieldType) do
      SL.Add(FFields[Field]);
    Result := SL.CommaText;
  finally
    SL.Free();
  end;
end;

procedure TDendoDBRecord.Put(Field: TDendoFieldType; Value: String);
begin
  FFields[Field] := Value;
end;

procedure TDendoDBRecord.SetText(Value: String);
var
  Field: TDendoFieldType;
  SL: TStringList;
begin
  SL := TStringList.Create();
  try
    SL.CommaText := Value;
    for Field := Low(TDendoFieldType) to High(TDendoFieldType) do
      FFields[Field] := SL[Ord(Field)];
  finally
    SL.Free();
  end;
end;

procedure TDendoDBRecord.Assign(Rec: TDendoDBRecord);
var
  Field: TDendoFieldType;
begin
  for Field := Low(TDendoFieldType) to High(TDendoFieldType) do
    FFields[Field] := Rec[Field];
end;

function TDendoDBRecord.Dup(): TDendoDBRecord;
begin
  Result := TDendoDBRecord.Create();
  Result.Assign(Self);
end;

{ TDendoDatabase }

function TDendoDatabase.GetItem(Index: Integer): TDendoDBRecord;
begin
  Result := TDendoDBRecord(inherited GetItem(Index));
end;

procedure TDendoDatabase.Assign(DB: TDendoDatabase);
var
  i: Integer;
begin
  Clear();
  for i := 0 to DB.Count - 1 do
    Add(DB[i].Dup());
end;

procedure TDendoDatabase.AssignToStrings(Strings: TStrings);
var
  SL: TStringList;
  i: Integer;
begin
  SL := TStringList.Create();
  try
    for i := 0 to Count - 1 do
      SL.Add(Items[i][dftState]);
    Strings.Assign(SL);
  finally
    SL.Free();
  end;
end;

function TDendoDatabase.IndexOf(Field: TDendoFieldType; Value: String): Integer;
var
  i: Integer;
begin
  Result := -1;
  for i := 0 to Count - 1 do
    if GetItem(i)[Field] = Value then
    begin
      Result := i;
      break;
    end;
end;

procedure TDendoDatabase.LoadFromFile(FileName: TFileName);
var
  SL: TStringList;
  Rec: TDendoDBRecord;
  i: Integer;
begin
  Clear();
  SL := TStringList.Create();
  try
    SL.LoadFromFile(FileName);
    for i := 0 to SL.Count - 1 do
    begin
      Rec := TDendoDBRecord.Create();
      Rec.Text := SL[i];
      Add(Rec);
    end;
  finally
    SL.Free();
  end;
end;

procedure TDendoDatabase.MergeDB(OldDB: TDendoDatabase);
var
  FileName: String;
  Index: Integer;
  i: Integer;
begin
  for i := 0 to OldDB.Count - 1 do
  begin
    FileName := OldDB[i][dftFileName];
    Index := IndexOf(dftFileName, FileName);
    if Index < 0 then // dendo armageddon ?
      if DendoStateStrs[dstDownloaded] = OldDB[i][dftState] then
        Add(OldDB[i].Dup()) // add only downloaded one
      else
    else // same key (filename) found then set download state
      Items[Index][dftState] := OldDB[i][dftState];
  end;
  SortBy(dftID, False);
end;

procedure TDendoDatabase.SaveToFile(FileName: TFileName);
var
  SL: TStringList;
  i: Integer;
begin
  SL := TStringList.Create();
  try
    for i := 0 to Count - 1 do
      SL.Add(Items[i].Text);
    SL.SaveToFile(FileName);
  finally
    SL.Free();
  end;
end;

var
  SortAscending: Boolean;
  SortField: TDendoFieldType;

function Compare(Item1, Item2: Pointer): Integer;
var
  Rec1, Rec2: TDendoDBRecord;
begin
  Rec1 := TDendoDBRecord(Item1);
  Rec2 := TDendoDBRecord(Item2);
  Result := CompareStr(Rec1[SortField], Rec2[SortField]);
  if 0 = Result then
    Result := CompareStr(Rec1[dftID], Rec2[dftID]);
  if not SortAscending then
    Result := - Result;
end;

procedure TDendoDatabase.SortBy(Field: TDendoFieldType; IsAscending: Boolean);
begin
  SortField := Field;
  SortAscending := IsAscending;
  Sort(@Compare);
end;

end.
